/*
 * ParametrizationTestDummy.java
 *
 * Created on 3. Mrz 2002, 19:47
 */

package de.tivano.junit;

import junit.framework.TestCase;
import java.util.Properties;
import java.io.File;
import java.lang.reflect.Field;

/**
 * Helper class to test the parametrized test suite. Basically, this class defines
 * parametrizable attributes of every allowable type, both accessible directly
 * and via method methods. In addition, it defines test methods to check if the
 * parametrized fields are set to the expected values.
 * @author  richard
 */
public class ParametrizationTestDummy extends TestCase {
    
    private Properties expectedData;
    private String keyPrefix;
    
    public boolean fieldBoolean;
    public char fieldChar;
    public byte fieldByte;
    public short fieldShort;
    public int fieldInt;
    public long fieldLong;
    public float fieldFloat;
    public double fieldDouble;
    public String fieldString;
    public File fieldFile;
    
    public Boolean fieldBooleanObj;
    public Character fieldCharObj;
    public Byte fieldByteObj;
    public Short fieldShortObj;
    public Integer fieldIntObj;
    public Long fieldLongObj;
    public Float fieldFloatObj;
    public Double fieldDoubleObj;
    
    private boolean methodBoolean;
    private char methodChar;
    private byte methodByte;
    private short methodShort;
    private int methodInt;
    private long methodLong;
    private float methodFloat;
    private double methodDouble;
    private String methodString;
    private File methodFile;
    
    private Boolean methodBooleanObj;
    private Character methodCharObj;
    private Byte methodByteObj;
    private Short methodShortObj;
    private Integer methodIntObj;
    private Long methodLongObj;
    private Float methodFloatObj;
    private Double methodDoubleObj;
    
    // This is a field that is accessible both by a setter and as a public field.
    // The setter is expected to set it to the negative of its argument, so
    // that we can use the value later to make sure that the setter really has been
    // called.
    public int special;
    
    // Don't set this in the associated properties - 
    // we're expecting it to be left alone...
    public String leaveMeAlone = "DONT_EVER_CHANGE_THIS_VALUE";
    
    
    public ParametrizationTestDummy(String name) {
        super(name);
    }
    
    /** Set the properties object and key prefix to check data against. */
    public void setExpectedData(String keyPrefix, Properties expectedData) {
        this.expectedData = expectedData;
        this.keyPrefix = keyPrefix;
    }
    
    /** Setter for property methodBoolean.
     * @param methodBoolean New value of property methodBoolean.
     */
    public void setMethodBoolean(boolean methodBoolean) {
        this.methodBoolean = methodBoolean;
    }
    
    /** Setter for property methodBooleanObj.
     * @param methodBooleanObj New value of property methodBooleanObj.
     */
    public void setMethodBooleanObj(java.lang.Boolean methodBooleanObj) {
        this.methodBooleanObj = methodBooleanObj;
    }
    
    /** Setter for property methodChar.
     * @param methodChar New value of property methodChar.
     */
    public void setMethodChar(char methodChar) {
        this.methodChar = methodChar;
    }
    
    /** Setter for property methodCharObj.
     * @param methodCharObj New value of property methodCharObj.
     */
    public void setMethodCharObj(java.lang.Character methodCharObj) {
        this.methodCharObj = methodCharObj;
    }
    
    /** Setter for property methodDouble.
     * @param methodDouble New value of property methodDouble.
     */
    public void setMethodDouble(double methodDouble) {
        this.methodDouble = methodDouble;
    }
    
    /** Setter for property methodDoubleObj.
     * @param methodDoubleObj New value of property methodDoubleObj.
     */
    public void setMethodDoubleObj(java.lang.Double methodDoubleObj) {
        this.methodDoubleObj = methodDoubleObj;
    }
    
    /** Setter for property methodFile.
     * @param methodFile New value of property methodFile.
     */
    public void setMethodFile(File methodFile) {
        this.methodFile = methodFile;
    }
    
    /** Setter for property methodFloat.
     * @param methodFloat New value of property methodFloat.
     */
    public void setMethodFloat(float methodFloat) {
        this.methodFloat = methodFloat;
    }
    
    /** Setter for property methodFloatObj.
     * @param methodFloatObj New value of property methodFloatObj.
     */
    public void setMethodFloatObj(java.lang.Float methodFloatObj) {
        this.methodFloatObj = methodFloatObj;
    }
    
    /** Setter for property methodInt.
     * @param methodInt New value of property methodInt.
     */
    public void setMethodInt(int methodInt) {
        this.methodInt = methodInt;
    }
    
    /** Setter for property methodIntObj.
     * @param methodIntObj New value of property methodIntObj.
     */
    public void setMethodIntObj(java.lang.Integer methodIntObj) {
        this.methodIntObj = methodIntObj;
    }
    
    /** Setter for property methodLong.
     * @param methodLong New value of property methodLong.
     */
    public void setMethodLong(long methodLong) {
        this.methodLong = methodLong;
    }
    
    /** Setter for property methodLongObj.
     * @param methodLongObj New value of property methodLongObj.
     */
    public void setMethodLongObj(java.lang.Long methodLongObj) {
        this.methodLongObj = methodLongObj;
    }
    
    /** Setter for property methodShort.
     * @param methodShort New value of property methodShort.
     */
    public void setMethodShort(short methodShort) {
        this.methodShort = methodShort;
    }
    
    /** Setter for property methodShortObj.
     * @param methodShortObj New value of property methodShortObj.
     */
    public void setMethodShortObj(java.lang.Short methodShortObj) {
        this.methodShortObj = methodShortObj;
    }
    
    /** Setter for property methodString.
     * @param methodString New value of property methodString.
     */
    public void setMethodString(java.lang.String methodString) {
        this.methodString = methodString;
    }

    /** Setter for property special.
     * This sets the <code>special</code> property to the negative of <code>value</code>.
     * @param value The value
     */
    public void setSpecial(int value) {
        // Set this to the negative value, so that we can check later that the
        // property really has been set by calling the setter method.
        this.special = -value;
    }
    
    /** Setter for property methodByteObj.
     * @param methodByteObj New value of property methodByteObj.
     */
    public void setMethodByteObj(java.lang.Byte methodByteObj) {
        this.methodByteObj = methodByteObj;
    }
    
    /** Setter for property methodByte.
     * @param methodByte New value of property methodByte.
     */
    public void setMethodByte(byte methodByte) {
        this.methodByte = methodByte;
    }
    
   /** Validate the "standard" data against the expected data. */
    public void testStandardData() throws IllegalAccessException {
        // Paranoia code: Make sure we do *have* the expected data...
        assertNotNull("Expected data not set", expectedData);
        assertNotNull("Key prefix not set", keyPrefix);
        // Go over all of the fields by reflection.
        Field[] allFields = getClass().getDeclaredFields();
        // Paranoia code: We've got 40 fields in this class at the time 
        // of writing this method, make sure we're getting them all...
        assertEquals("ParametrizationTestDummy.testStandardData(), " + keyPrefix + 
                     ": Field count mismatch:", 40, allFields.length);
        for (int i=0; i<allFields.length; i++) {
            String name = allFields[i].getName();
            if (name.startsWith("field") || name.startsWith("method")) {
                String value = allFields[i].get(this).toString();
                String expected = expectedData.getProperty(keyPrefix + "." + name);
                assertEquals("ParametrizationTestDummy.testStandardData(), " + keyPrefix + 
                             ": Data for property '" + name + "':", expected, value);
            }
        }
    }
    
    /** Test if the paramterization really prefers the setter to a public field. */
    public void testSetterMethodAndPublicField() {
        // Paranoia code: Make sure we do *have* the expected data...
        assertNotNull("ParametrizationTestDummy.testSetterMethodAndPublicField(): Expected data not set:", expectedData);
        assertNotNull("ParametrizationTestDummy.testSetterMethodAndPublicField(): Key prefix not set", keyPrefix);
        int expected = -Integer.parseInt(expectedData.getProperty(keyPrefix + ".special"));
        assertEquals("ParametrizationTestDummy.testSetterMethodAndPublicField(), " + keyPrefix + ":", 
                     expected, special);
    }
    
    /** Test if the "leave me alone" field has indeed been left alone by the parametrization... */
    public void testLeaveMeAlone() {
        assertEquals("ParametrizationTestDummy.testLeaveMeAlone(): ", "DONT_EVER_CHANGE_THIS_VALUE", leaveMeAlone);
    }
    
    /** A test that always fails. Needed to check that the parametrization decorator actually calls
     * the decorated test.
     */
    public void testFailAlways() {
        fail("ParametrizationTestDummy.testFailAlways(): This test is expected to fail");
    }
 }
