/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LazyHTMLDOMImplementation.java,v 1.1.1.1 2003/03/10 16:36:18 taweili Exp $
 */

package org.enhydra.xml.lazydom.html;

import org.enhydra.xml.lazydom.LazyDOMImplementation;
import org.enhydra.xml.lazydom.TemplateDOM;
import org.w3c.dom.html.HTMLDOMImplementation;
import org.w3c.dom.html.HTMLDocument;
import org.w3c.dom.DOMException;

/**
 * HTMLDOMImplementation method for the LazyDOM.

 * @see org.w3c.dom.html.HTMLDOMImplementation
 */
public class LazyHTMLDOMImplementation extends LazyDOMImplementation 
    implements HTMLDOMImplementation {

    /** DOM implementation singleton. */
    private static LazyHTMLDOMImplementation fSingleton;

    /**
     * Obtain and return the singleton shared object.
     */
    public static HTMLDOMImplementation getHTMLDOMImplementation() {
        if (fSingleton == null) {
            fSingleton = new LazyHTMLDOMImplementation();
        }
        return fSingleton;
    }  
    
    /**
     * Create a new LazyHTMLDocument.
     * @see org.w3c.dom.html.HTMLDOMImplementation#createHTMLDocument
     */
    public HTMLDocument createHTMLDocument(String title) {
	HTMLDocument doc = new LazyHTMLDocument();
	doc.setTitle(title);
	return doc;
    }

    /**
     * Create a new LazyHTMLDocument from a template..
     * @see org.w3c.dom.html.HTMLDOMImplementation#createHTMLDocument
     */
    public HTMLDocument createHTMLDocument(TemplateDOM templateDOM) {
	return new LazyHTMLDocument(templateDOM);
    }
}
