/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s): 
 * Richard Kunze
 * $Id: DocumentLoader.java,v 1.1.1.1 2003/03/10 16:36:21 taweili Exp $
 */

package org.enhydra.xml.xmlc.deferredparsing;

import org.w3c.dom.Document;
import org.enhydra.xml.xmlc.XMLCRuntimeException;

/**
 * Interface for creating a DOM by document name. This interface is
 * used by XMLC generated classes with deferred parsing support to
 * delegate the actual work of DOM generation.  Note: Document loaders
 * may be shared between different XMLC classes and accessed
 * concurrently. Classes implementing this interface must be thread
 * safe (reentrant preferred).
 */
public interface DocumentLoader {

    /**
     * Bind to a factory.  This is done so that loggers, etc
     * can be grabbed from the factory.
     *
     * @param factory Factory that is creating the object.  Useful
     *  for getting logging object.
     */
    public void init(XMLCDeferredParsingFactory factory);

    /** 
     * Get a DOM instance representing the specified document.
     * @param docClass the XMLC generated class to get the DOM for.
     * @return the DOM representing objects of <code>docClass</code>
     */
    public Document getDocument(Class docClass) throws XMLCRuntimeException;
}
