/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LazyHTMLDomFactory.java,v 1.1.1.1 2003/03/10 16:36:22 taweili Exp $
 */

package org.enhydra.xml.xmlc.dom.lazydom;

import org.enhydra.xml.xmlc.dom.XMLCDomFactory;
import org.enhydra.xml.xmlc.dom.HTMLDomFactoryMethods;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.enhydra.xml.lazydom.html.LazyHTMLDocument;
import org.enhydra.xml.lazydom.TemplateDOM;

/**
 * Lazy DOM factory for creating HTML Document objects.
 */
public class LazyHTMLDomFactory extends LazyDomFactory
    implements XMLCDomFactory {

    /**
     * @see XMLCDomFactory#createDocument
     */
    public Document createDocument(String namespaceURI,
                                   String qualifiedName,
                                   DocumentType docType) {
        LazyHTMLDocument doc = new LazyHTMLDocument(null);
        // This creates the document element.
        doc.getDocumentElement();
        return doc;
    }

    /**
     * Create a LazyDocument from a template.
     * @see XMLCDomFactory#createDocument
     */
    public Document createDocument(TemplateDOM templateDOM) {
        return new LazyHTMLDocument(templateDOM);
    }

    /**
     * @see XMLCDomFactory#getMIMEType
     */
    public String getMIMEType() {
        return "text/html";
    }

    /**
     * @see XMLCDomFactory#getBaseClassName
     */
    public String getBaseClassName() {
        return HTMLDomFactoryMethods.getBaseClassName();
    }

    /**
     * @see XMLCDomFactory#getInterfaceNames
     */
    public String[] getInterfaceNames() {
        return HTMLDomFactoryMethods.getInterfaceNames();
    }

    /**
     * @see XMLCDomFactory#getElementClassNames
     */
    public String[] getElementClassNames(Element element) {
        return HTMLDomFactoryMethods.getElementClassNames(element);
    }

    /**
     * @see XMLCDomFactory#isURLAttribute
     */
    public boolean isURLAttribute(Element element,
                                  String attrName) {
        return HTMLDomFactoryMethods.isURLAttribute(element, attrName);
    }
}
