/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XercesNodeCreateGenerator.java,v 1.1.1.1 2003/03/10 16:36:22 taweili Exp $
 */

package org.enhydra.xml.xmlc.dom.xerces;

import org.enhydra.xml.dom.DOMOps;
import org.enhydra.xml.xmlc.XMLCError;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.dom.XMLCDocument;
import org.enhydra.xml.xmlc.codegen.JavaLang;
import org.enhydra.xml.xmlc.codegen.JavaCode;
import org.enhydra.xml.xmlc.dom.generic.NodeCreateGenerator;
import org.w3c.dom.*;
import org.enhydra.xml.xmlc.compiler.ClassGenerator;

/**
 * Class used create DOM nodes for the Xerces DOM.
 */
public class XercesNodeCreateGenerator extends NodeCreateGenerator {
    /**
     * Constructor.
     */
    public XercesNodeCreateGenerator(XMLCDocument xmlcDoc) {
        super(xmlcDoc);
    }

    /**
     * Generate Entity creation.
     */
    protected void genEntityCreate(Entity entity,
                                   String docVar,
                                   String varName, 
                                   JavaCode body) {
        body.addln(varName + " = " + docVar + ".createEntity("
                   + JavaLang.createStringConst(entity.getNodeName()));
        body.addln(varName + ".setPublicId("
                   + JavaLang.createStringConst(entity.getPublicId()) + ");");
        body.addln(varName + ".setSystemId("
                   + JavaLang.createStringConst(entity.getSystemId()) + ");");
	try {
	    String encoding = DOMOps.getEncoding(entity);
	    body.addln(
	       "try { " +
	           "DOMOps.setEncoding(" + varName + ", " + encoding +"); " +
	       "} catch (UnsupportedOperationException e) { /* ignore */ }"
	   );
	} catch (UnsupportedOperationException e) {
	    // Ignore this - if we can't read the encoding here, no point
	    // in trying to set it in the compiled document...
	}
	    
        body.addln(varName + ".setNotationName("
                   + JavaLang.createStringConst(entity.getNotationName())
		   + ");");
    }

    /**
     * Generate Notation creation.
     */
    protected void genNotationCreate(Notation notation,
                                     String docVar,
                                     String varName, 
                                     JavaCode body) {
        body.addln(varName + " = " + docVar + ".createNotation("
                   + JavaLang.createStringConst(notation.getNodeName()));
        body.addln(varName + ".setPublicId("
                   + JavaLang.createStringConst(notation.getPublicId()) + ");");
        body.addln(varName + ".setSystemId("
                   + JavaLang.createStringConst(notation.getSystemId()) + ");");
    }

    /**
     * Add an entity to a DocumentType
     */
    public void genAddEntity(String docTypeVar,
                             String entityVar,
                             Entity entity,
                             JavaCode body) {
        //FIXME: should be in xerces version of this file
        if (entity.getNamespaceURI() != null) {
            body.addln(docTypeVar + ".getEntities().setNamedItemNS("
                       + entityVar + ");");
        } else {
            body.addln(docTypeVar + ".getEntities().setNamedItem("
                       + entityVar + ");");
        }
    }

    /**
     * Add an notation to a DocumentType
     */
    public void genAddNotation(String docTypeVar,
                               String notationVar,
                               Notation notation,
                               JavaCode body) {
        //FIXME: should be in xerces version of this file
        if (notation.getNamespaceURI() != null) {
            body.addln(docTypeVar + ".getNotations().setNamedItemNS("
                       + notationVar + ");");
        } else {
            body.addln(docTypeVar + ".getNotations().setNamedItem("
                       + notationVar + ");");
        }
    }
}
