/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DocumentSection.java,v 1.1.1.1 2003/03/10 16:36:23 taweili Exp $
 */

package org.enhydra.xml.xmlc.metadata;

import org.w3c.dom.*;

/**
 * Section that describes the structure of the document and attributes of the
 * elements.  This information is used to validate a document and to control
 * code generate on specific elements in the document.
 */
public class DocumentSection extends MetaDataElement {
    /**
     * Element name.
     */
    public static final String TAG_NAME = "document";

    /**
     * Constructor.
     */
    public DocumentSection(Document ownerDoc) {
    	super(ownerDoc, TAG_NAME);
    }

    /**
     * Get the ElementDef child elements.
     */
    public ElementDef[] getElementDef() {
        return (ElementDef[])getChildren(ElementDef.class);
    }

    /**
     * Add a ElementDef child element.
     */
    public void addElementDef(ElementDef elementDef) {
        appendChild(elementDef);
    }

    /**
     * Delete a ElementDef child element.
     */
    public void deleteElementDef(ElementDef elementDef) {
        removeChild(elementDef);
    }

    /**
     * Get the TagClass child elements.
     */
    public TagClass[] getTagClass() {
        return (TagClass[])getChildren(TagClass.class);
    }

    /**
     * Add a TagClass child element.
     */
    public void addTagClass(TagClass tagClass) {
        appendChild(tagClass);
    }

    /**
     * Delete a TagClass child element.
     */
    public void deleteTagClass(TagClass tagClass) {
        removeChild(tagClass);
    }
}
