/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLCyberStudioTagSet.java,v 1.1.1.1 2003/03/10 16:36:23 taweili Exp $
 */

package org.enhydra.xml.xmlc.metadata;

/**
 * Abobe CyberStudio proprietary tags.  CyberStudio makes up a bunch of tags
 * it expects to be ignored on display.  It uses these for metadata and its
 * all rather tacky.  This returns lists of HTMLTagDef and HTMLAttrDef
 * objects. It is not part of the DOM.  Objects are immutable and a singleton.
 */
class HTMLCyberStudioTagSet extends HTMLTagSetDef {
    /**
     * Singleton instance.
     */
    private static HTMLCyberStudioTagSet cyberStubioTagSet;

    /**
     * Constructor.
     */
    private HTMLCyberStudioTagSet() {
        String[] csBlockTags = {
            "csactionclass", "csactiondict", "csactionitem",
            "csactionparam", "csactions",    "csgroup",
            "csimport",      "csmagic",      "cspart",
            "csscriptdict",  "cssequence",   "cssequencer",
            "cstrack"
        };
        for (int i = 0; i < csBlockTags.length; i++) {
            // block
            addTagDef(new HTMLTagDefImpl(csBlockTags[i], false, true,
                                         false, false));
        }

        // block+inline
        addTagDef(new HTMLTagDefImpl("csobj", true, true, false, false));

        String[] csEmptyTags = {
            "csbrowser",    "csaction",  "csactioncntl",
            "csactionenum", "csgeneric", "csseqaction"
        };
        for (int i = 0; i < csEmptyTags.length; i++) {
            // inline+empty+optclose
            addTagDef(new HTMLTagDefImpl(csEmptyTags[i], true, false,
                                         true, true));
        }
        setReadOnly();
    }

    /**
     * Get the singleton instance of the tag set.
     */
    public static HTMLCyberStudioTagSet getTagSet() {
        if (cyberStubioTagSet == null) {
            cyberStubioTagSet = new HTMLCyberStudioTagSet();
        }
        return cyberStubioTagSet;
    }
}
