/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLTagSet.java,v 1.1.1.1 2003/03/10 16:36:23 taweili Exp $
 */

package org.enhydra.xml.xmlc.metadata;

import org.enhydra.xml.xmlc.XMLCError;
import org.w3c.dom.Document;

/**
 * Add a predefined set of proprietary HTML tag and attributes to the list of
 * valid HTML tags. This option is only used by the <CODE>tidy</CODE> parser.
 */
public class HTMLTagSet extends MetaDataElement {
    /**
     * Element name.
     */
    public static final String TAG_NAME = "htmlTagSet";

    /**
     * Attribute names.
     */
    private static final String TAGSET_ATTR = "tagSet";

    /**
     * Constructor.
     */
    public HTMLTagSet(Document ownerDoc) {
    	super(ownerDoc, TAG_NAME);
    }

    /**
     * Get the tagSet to use.
     */
    public HTMLTagSetType getTagSet() {
        return HTMLTagSetType.getType(getAttributeNull(TAGSET_ATTR));
    }

    /**
     * Set the tagSet to use.
     */
    public void setTagSet(HTMLTagSetType value) {
        if (value == null) {
            removeAttribute(TAGSET_ATTR);
        } else {
            setAttribute(TAGSET_ATTR, value.getName());
        }
    }

    /**
     * Get the HTMLTagSetDef object for the tag set in the element.
     */
    public HTMLTagSetDef getTagSetDef() {
        HTMLTagSetType tagSet = getTagSet();
        if (tagSet == null) {
            throw new XMLCError("HTML tag set not defined");
        } else if (tagSet == HTMLTagSetType.CYBERSTUDIO) {
            return HTMLCyberStudioTagSet.getTagSet();
        } else {
            throw new XMLCError("Bug: Unknown HTML tag set");
        }
    }
}
