/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLTagSetDef.java,v 1.1.1.1 2003/03/10 16:36:23 taweili Exp $
 */

package org.enhydra.xml.xmlc.metadata;

import java.util.ArrayList;

/**
 * Base class used for defining a proprietary set of HTML tags.
 */
abstract class HTMLTagSetDef {
    /**
     * Tag and attribute definitions.
     */
    private ArrayList tagDefs = new ArrayList();
    private ArrayList attrDefs = new ArrayList();

    /**
     * Is set completed (can't modify);
     */
    private boolean readOnly = false;

    /**
     * Check if class can be modified.
     */
    private void modifyCheck() {
        if (readOnly) {
            throw new UnsupportedOperationException("HTMLTagSetDef object is read-only");
        }
    }

    /**
     * Add an tag definition.
     */
    protected void addTagDef(HTMLTagDef tagDef) {
        modifyCheck();
        tagDefs.add(tagDef);
    }

    /**
     * Add an attribute definition.
     */
    protected void addAttrDef(HTMLAttrDef attrDef) {
        modifyCheck();
        attrDefs.add(attrDef);
    }
    /**
     * Mark readonly.
     */
    protected void setReadOnly() {
        readOnly = true;
    }

    /**
     * Get the tag definitions.
     */
    public HTMLTagDef[] getTagDefs() {
        return (HTMLTagDef[])tagDefs.toArray(new HTMLTagDef[tagDefs.size()]);
    }

    /**
     * Get the attribute definitions.
     */
    public HTMLAttrDef[] getAttrDefs() {
        return (HTMLAttrDef[])attrDefs.toArray(new HTMLAttrDef[attrDefs.size()]);
    }
}
