/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BasicTests.java,v 1.1.1.1 2003/03/10 16:36:26 taweili Exp $
 */

package org.enhydra.xml.lazydom;

import org.enhydra.xml.xmlc.driver.XmlcTestParams;
import java.lang.reflect.Method;
import org.enhydra.xml.driver.TestException;
import org.enhydra.xml.driver.TestDomOps;
import org.enhydra.xml.lazydom.LazyDocument;
import org.enhydra.xml.lazydom.LazyElement;
import org.enhydra.xml.io.DOMParser;
import org.enhydra.xml.dom.DOMInfo;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import java.lang.reflect.Method;
import java.io.File;
import junit.framework.Test;

/**
 * Basic tests of Lazy DOM.  The XMLC tests do good job of testing a lot
 * of the LazyDOM. This concentrates on corner cases and bug regression.
 * There really needsto be a lot more here.
 */
public class BasicTests extends LazyDOMTestCaseBase {
    /** Input files */
    private static final String INSULIN3_XML_DIR = "../xmlc/xml";
    private static final String INSULIN3_XML = "bioml/insulin3.bioml";

    /** DOM builder and parser */
    private LazyDOMBuilder fBuilder
        = new LazyDOMBuilder(getMsgWriter(),
                             org.enhydra.xml.xmlc.dom.lazydom.LazyDomFactory.class);

    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(BasicTests.class, null);
    }
    
    /** Constructor */
    public BasicTests(Method method) {
        super(method);
    }

    /** Create an unexpanded document from insulin3.bioml */
    private LazyDocument parseInsulin3() {
        File insulin3 = getOtherInputFile(INSULIN3_XML_DIR, INSULIN3_XML);
        return fBuilder.parseUnexpanded(insulin3);
    }

    /**
     * Test 1: Simple test on insulin3.bioml
     */
    public void test1() {
        LazyDocument lazyDoc = parseInsulin3();
        printDiffDom("Unexpanded Lazy DOM", lazyDoc, "unexpanded.dom");
        TestDomOps.expandTree(lazyDoc);
        printDiffDom("Expanded Lazy DOM", lazyDoc, "expanded.dom");
    }

    /**
     * Basic modifications on insulin3.bioml, 
     */
    public void test2() {
        LazyDocument lazyDoc = parseInsulin3();
        DocInfo docInfo = new DocInfo(lazyDoc);
        
        // FIXME: this was disabled
        // Add a text node to an element.  
        Element lazyElem = docInfo.getByAttrValue(lazyDoc, "label",
                                                  "HUMINS locus");
        lazyElem.appendChild(lazyDoc.createTextNode("@ new node #1 @"));

        // Add a attribute
        lazyElem = docInfo.getByAttrValue(lazyDoc, "end", "4992");
        lazyElem.setAttribute("newAttr", "new value");

        // Expand the parent
        lazyElem = docInfo.getByAttrValue(lazyDoc, "format", "PIR");

        Node parent = (Element)lazyElem.getParentNode();
        String name = parent.getNodeName();  // Make sure we got something
        
        printDiffDom("Modified, unexpanded Lazy DOM", lazyDoc,
                     "unexpanded.dom");
        TestDomOps.expandTree(lazyDoc);
        printDiffDom("Modified, expanded Lazy DOM", lazyDoc,
                     "expanded.dom");
    }

    /**
     * Test 3: cloning of attribute on shallow clone.  This failed at one
     * time.
     */
    public void test3() {
        String tagName = "organism";
        LazyDocument lazyDoc = parseInsulin3();
        DocInfo docInfo = new DocInfo(lazyDoc);

        // Find an Element, without attributes expanded
        LazyElement elem = (LazyElement)getElementByTagName(lazyDoc, lazyDoc, tagName);
        if (elem == null) {
            throw new TestException("element null for tag: " + tagName);
        }
        if (elem.areAttributesExpanded()) {
            throw new TestException("attributes should not be expanded");
        }
        LazyElement cloneElem = (LazyElement)elem.cloneNode(false);
        if (!elem.areAttributesExpanded()) {
            new TestException("attributes should be expanded");
        }
        String labelValue = cloneElem.getAttribute("label");
        if ((labelValue == null) || !labelValue.equals("Homo sapiens (human)")) {
            throw new TestException("attribute \"label\" has wrong value: \""
                                    + labelValue + "\"");
        }
    }
}
