/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LazyDOMTestCaseBase.java,v 1.1.1.1 2003/03/10 16:36:26 taweili Exp $
 */

package org.enhydra.xml.lazydom;

import org.enhydra.xml.driver.TestCaseBase;
import java.lang.reflect.Method;
import org.enhydra.xml.driver.TestException;
import org.enhydra.xml.driver.TestDiff;
import org.enhydra.xml.driver.TestFileOps;
import org.enhydra.xml.dom.DOMAccess;
import org.enhydra.xml.dom.DOMInfo;
import org.enhydra.xml.dom.DOMStats;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import java.io.File;
import java.io.PrintWriter;
import java.io.FileWriter;

/**
 * Base class for lazy dom test cases.  Contains common code.
 */
public class LazyDOMTestCaseBase extends TestCaseBase {
    /** Constructor. */
    protected LazyDOMTestCaseBase(Method method) {
        super(method);
    }

    /**
     * Print DOM to a file, starting at a node and diff with expected
     */
    protected void printDiffDom(String msg,
                                Node node,
                                String ext) {
        File outFile = getResultFile(ext);
        File expectFile = getExpectedFile(ext);
        TestFileOps.ensureFileDir(outFile);

        try {
            PrintWriter out = new PrintWriter(new FileWriter(outFile));
            try {
                DOMInfo.printTree(msg, node, 
                                  DOMInfo.PRINT_DEFAULT|DOMInfo.PRINT_ATTR_DETAILS,
                                  out);
                DOMStats.printStats(msg, node, 0, out);
            } finally {
                out.close();
            }
        } catch (Exception except) {
            throw new TestException(except);
        }
        getDiffer().diff(expectFile, outFile);
    }

    /**
     * Recursively find an element-by-tag-name without expanding.
     */
    private Element accessElementByTagName(DOMAccess accessor,
                                           Node root,
                                           String tagName) {
        if ((root instanceof Element)
            && (((Element)root).getTagName().equals(tagName))) {
            return (Element)root;
        } else {
            Element foundNode = null;
            for (Node child = accessor.accessFirstChild(root);
                 (child != null) && (foundNode == null);
                 child = accessor.accessNextSibling(child)) {
                foundNode = accessElementByTagName(accessor, child, tagName);
            }
            return foundNode;
        }
    }

    /**
     * Find the first element of the specified tag, but do no expansion except
     * for the returned node.
     */
    public Element getElementByTagName(Document doc,
                                       Node root,
                                       String tagName) {
        DOMAccess accessor = new DOMAccess (doc);
        Element element = accessElementByTagName(accessor, root, tagName);
        if (element != null) {
            element = accessor.getExpandedElement(element);
        }
        return element;
    }
}
