/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BasicTests.java,v 1.1.1.1 2003/03/10 16:36:28 taweili Exp $
 */

package org.enhydra.xml.xmlc.html;
import org.enhydra.xml.driver.TestDiff;
import org.enhydra.xml.xmlc.driver.XmlcTestParams;
import org.enhydra.xml.xmlc.driver.ExecXmlc;
import org.enhydra.xml.io.OutputOptions;
import java.io.File;
import java.lang.reflect.Method;
import junit.framework.Test;

/**
 * Tests of basic HTML functionality.
 */
public class BasicTests extends HtmlTestCaseBase {

    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(BasicTests.class, null);
    }
    
    /** Constructor */
    public BasicTests(Method method) {
        super(method);
    }

    /** 
     * Test 1: compile of HTML with non-standard tags and using
     * an options file.
     */
    public void test1() {
        HtmlBasicTest test = createLutrisIndexTest();
        test.addOptionFile(getInputFile("lutris-index1.xmlc"));
        test.basicTest();
    }

    /*
     * Test 2: -urlmapping and -g option
     */
    public void test2() {
        HtmlBasicTest test = createLutrisIndexTest();
        test.addBoth(ExecXmlc.OPT_JAVAC_G);
        test.addFirst(ExecXmlc.OPT_URL_MAPPING);
        test.addFirst("TheCompany1.html", "TheCompany1.po");
        test.addFirst(ExecXmlc.OPT_URL_MAPPING);
        test.addFirst("ProfSrvcs1.html", "ProfSrvcs1.po");
        test.setUseToDocument();
        test.basicTest();
    }

    /*
     * Test 3: -urlsetting and -g option
     */
    public void test3() {
        HtmlBasicTest test = createTest("simple-href.html");
        test.addFirst(ExecXmlc.OPT_URL_SETTING);
        test.addFirst("testNode", "http://www.foo.com/");
        test.basicTest();
    }

    /*
     * Test 5: -urlregexpmapping in an option file
     */
    public void test5() {
        HtmlBasicTest test = createLutrisIndexTest();
        test.addOptionFile(getInputFile("LutrisIndex4.xmlc"));
        test.setUseToDocument();
        test.basicTest();
    }

    /*
     * Test 6: File containing java script and a noscript tag.
     */
    public void test6() {
        HtmlBasicTest test = createTest("CheckVersion.html");
        test.basicTest();
    }

    /*
     * Test 7: Two -urlregexpmapping  
     */
    public void test7() {
        HtmlBasicTest test = createLutrisIndexTest();
        test.addBoth(ExecXmlc.OPT_JAVAC_G);
        test.addFirst(ExecXmlc.OPT_URL_REGEXP_MAPPING);
        test.addFirst("^(Th.+)\\.html$$", "$$1.po");
        test.addFirst(ExecXmlc.OPT_URL_REGEXP_MAPPING);
        test.addFirst("^(Pr.+)\\.html$$", "$$1.po");
        test.basicTest();
    }

    /*
     * Test 8: handling of special characters.  This contains HTML 4.0
     * additions, so the result will not be the same in every browser.
     */
    public void test8() {
        HtmlBasicTest test = createTest("special-char.html");
        test.basicTest();
    }

    /*
     * Test 9: Test handling of special characters in attribute values,
     * without using attribute entity references.
     */
    public void test9() {
        HtmlBasicTest test = createTest("attributeValues.html");
        test.getOutputOptions().setOmitAttributeCharEntityRefs(true);
        test.basicTest();
        File htmlFile = test.getGenerateFile();

        TestDiff differ = getDiffer();
        differ.fgrepMustFind("http://www.foo.bar/cgi-bin/pig3?a=b&c=d",
                             htmlFile);
        differ.fgrepMustFind("http://www.foo.bar/cgi-bin/pig4?a=&#34;&c=d",
                             htmlFile);
        differ.fgrepMustFind("http://www.foo.bar/cgi-bin/pig5?a=&#34;&c=d",
                             htmlFile);
    }

    /*
     * Test 10: Test handling of special characters in attribute values,
     * using attribute entity references.
     */
    public void test10() {
        HtmlBasicTest test = createTest("attributeValues.html");
        test.getOutputOptions().setOmitAttributeCharEntityRefs(false);
        test.basicTest();

        File htmlFile = test.getGenerateFile();
        TestDiff differ = getDiffer();
        differ.fgrepMustFind("http://www.foo.bar/cgi-bin/pig3?a=b&amp;c=d",
                             htmlFile);
        differ.fgrepMustFind("http://www.foo.bar/cgi-bin/pig4?a=&quot;&amp;c=d",
                             htmlFile);
        differ.fgrepMustFind("http://www.foo.bar/cgi-bin/pig5?a=&quot;&amp;c=d",
                             htmlFile);
    }

    /*
     * Test 11: Handling framesets
     */
    public void test11() {
        HtmlBasicTest test = createTest("FrameSet.html");
        test.basicTest();
    }
}
